import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.awt.image.*;
import java.io.File;

public class DrawGeometricMap {
    
    public static void main(PictureCanvas P,Graphics2D g) {
       	drawParallelogram(P,g);
	drawmapGuidelines(P,g);
	drawMapTriangle1(P,g);
	drawMapTriangle2(P,g);
    }


    /**This is the map construction*/
    public static void drawParallelogram(PictureCanvas P,Graphics2D g) {
     int j=P.SELECT;
     Complex[] Z=Triangulation.scaffold1(j,P.SOURCE);
     for(int i=0;i<3;++i) Z[i]=fix(Z[i]);
     Complex[] W=Triangulation.scaffold2(j,P.SOURCE);
     Complex[] ZW={Z[0],Z[1],W[0],W[1]};
     Path2D.Double gp=GraphicsHelp.toPath(ZW);
     gp=P.transform(gp);
     g.setColor(ColorCanvas.C[3][0]);
     g.fill(gp);
     g.setColor(ColorCanvas.C[3][1]);
     g.draw(gp);
  }

    /**This draws one of the triangles for the AFP*/
    public static void drawMapTriangle1(PictureCanvas P,Graphics2D g) {
	Color C=ColorCanvas.C[3][2];
	int j=P.SELECT;
	Complex[] Z=Triangulation.scaffold1(j,P.SOURCE);
	for(int i=0;i<3;++i) Z[i]=fix(Z[i]);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<3;++i) {
	    int j1=(i+1)%3;
	    int j2=(i+2)%3;
	    Complex[] W0={Z[j1],Z[j2]};
	    GraphicsHelp.draw(g,P,W0,C,1);
	}
    }

    /**This draws the other triangle for the AFP*/

    public static void drawMapTriangle2(PictureCanvas P,Graphics2D g) {
	int j=P.SELECT;
	Color C=ColorCanvas.C[3][3];
	Path2D.Double gp=new Path2D.Double(); 
        g.setStroke(new BasicStroke(1));
        Complex[] W=Triangulation.scaffold2(j,P.SOURCE);
	Complex[] Z={W[0],W[1],P.SOURCE};
	for(int i=0;i<3;++i) {
	    int j1=(i+1)%3;
	    int j2=(i+2)%3;
	    Complex[] W0={Z[j1],Z[j2]};
	    GraphicsHelp.draw(g,P,W0,C,1);
	}
    }


    /**This draws the guidelines for defining the AFP*/

    public static void drawmapGuidelines(PictureCanvas P,Graphics2D g) {
	int j=P.SELECT;
	Path2D.Double gp=new Path2D.Double();
        Complex[] Z=Triangulation.scaffold2(j,P.SOURCE);
        Complex[] H=Triangulation.scaffold1(j,P.SOURCE);
	for(int i=0;i<3;++i) H[i]=fix(H[i]);
	Color COL=new Color(255,140,0);
	int[] list1={0,1,4,5,8,9};
	int[] list2={2,1,0,2,1,0,2,1,0,2,1,0};
	boolean test=false;
	for(int i=0;i<6;++i) if(list1[i]==j) test=true;	 

	if(test==true) {
	    Color C2=ColorCanvas.C[3][2];
	    Color C1=ColorCanvas.C[3][3];
	  Path2D.Double gp2=Triangulation.tri(0,new Complex(0,0));
	  Complex[][] W=Triangulation.specialSegments();
	  for(int i=6;i<9;++i) {
	     Complex s=Vector.findCross2(P.SOURCE,Z[j%2],W[i][0],W[i][1]);
	     Complex[] ZZ={s,P.SOURCE};
	     GraphicsHelp.draw(g,P,ZZ,C1,1);  
	  }

	  int j2=list2[j];
	  Complex s=Vector.findCross2(P.ANTI,H[1-j%2],W[j2][0],W[j2][1]);
	  Complex[] ZZ={s,P.ANTI};
          GraphicsHelp.draw(g,P,ZZ,C2,1);
	}

	if(test==false) {
	    Color C2=ColorCanvas.C[3][2];
	    Color C1=ColorCanvas.C[3][3];
	  Path2D.Double gp2=Triangulation.tri(0,new Complex(0,0));
	  Complex[][] W=Triangulation.specialSegments();
	  int j2=list2[j];
	  Complex s=Vector.findCross2(P.SOURCE,Z[1-j%2],W[j2][0],W[j2][1]);
	  Complex[] ZZ={s,P.SOURCE}; 
          GraphicsHelp.draw(g,P,ZZ,C1,1);

          for(int i=6;i<9;++i) {
	     s=Vector.findCross2(P.ANTI,H[j%2],W[i][0],W[i][1]);
	     Complex[] ZZZ={s,P.ANTI};
             GraphicsHelp.draw(g,P,ZZZ,C2,1);	
	  }
	}
    }


    public static void drawMap(PictureCanvas P,Graphics2D g) {
	Color C=Color.blue;
	P.drawPoint(g,P.SOURCE,C,3);
	P.drawPoint(g,P.ANTI,C,3);
	Complex[] W={P.SOURCE,P.ANTI};
	GraphicsHelp.draw(g,P,W,Color.black,2);
    }


    public static Complex fix(Complex z) {
        Complex w=Octahedron.anti(z);
	w=Complex.conjugate(w);
	w=Complex.times(w,Octahedron.cr(2));
	return(w);
    }

}
