import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class Triangulation {

    public static Complex cr(int j) {
        double c=Math.cos(2*j*Math.PI/3);
	double s=Math.sin(2*j*Math.PI/3);
	Complex u=new Complex(c,s);
	return(u);
    }

    public static Complex[] corridor(int k) {
	 Complex[] w=new Complex[3];
         double c=Math.cos(Math.PI/3);
	 double s=Math.sin(Math.PI/3);
	 if(k==1) s=-s;
	 w[0]=new Complex(c,s);
	 w[1]=new Complex(0,2*s);
	 w[2]=new Complex(c,3*s);
	 return(w);
    }

    public static Path2D.Double tri(int k,Complex z) {
	Complex[] w=triPoints(k,z);
	Path2D.Double gp=GraphicsHelp.toPath(w);
	return(gp);
    }

   /**slightly enlarged version*/
    public static Path2D.Double triFat(int k,Complex z) {
	Complex[] w=triPointsFat(k,z);
	Path2D.Double gp=GraphicsHelp.toPath(w);
	return(gp);
    }

    public static Complex[] triPoints(int k,Complex z) {
	Complex[] w=tri(k);
	for(int i=0;i<3;++i) w[i]=Complex.plus(z,w[i]);
	return(w);
    }

    /**slightly enlarged version*/
    public static Complex[] triPointsFat(int k,Complex z) {
	Complex[] w=tri(k);
	for(int i=0;i<3;++i) w[i]=Complex.plus(z,w[i]);
	Complex c=average(w);
	double t=1.000001;
	for(int i=0;i<3;++i) {
	    w[i].x=t*w[i].x+(1-t)*c.x;
	    w[i].y=t*w[i].y+(1-t)*c.y;
	}
	return(w);
    }

    public static Complex average(Complex[] w) {
	Complex z=new Complex(0,0);
	for(int i=0;i<w.length;++i) z=Complex.plus(z,w[i]);
	z.x=z.x/w.length;
	z.y=z.y/w.length;
	return(z);
    }


    public static Complex[] tri(int k) {
	if(k==0) return(tri0());
	return(tri1());
    }


    public static Complex[] tri0() {
	Complex[] z=new Complex[3];
	for(int i=0;i<3;++i) {
	    double c=Math.cos(2*Math.PI*i/3);
	    double s=Math.sin(2*Math.PI*i/3);
	    z[i]=new Complex(c,s);
	}
	return(z);
    }


    public static Complex[] tri1() {
	Complex[] z=new Complex[3];
	double d=Math.PI/3;
	for(int i=0;i<3;++i) {
	    double c=Math.cos(d+2*Math.PI*i/3);
	    double s=Math.sin(d+2*Math.PI*i/3);
	    z[i]=new Complex(c,s);
	}
	return(z);
    }

    public static Path2D.Double tri(int i,int j,int k) {
	Complex[] w=corridor(k); 
	int[] m={1,0,1};
	Complex u=cr(j);
	Complex v=Complex.times(u,w[i]);
	Path2D.Double gp=tri(m[i],v);
	return(gp);
    }

    public static Path2D.Double triFat(int i,int j,int k) {
	Complex[] w=corridor(k); 
	int[] m={1,0,1};
	Complex u=cr(j);
	Complex v=Complex.times(u,w[i]);
	Path2D.Double gp=triFat(m[i],v);
	return(gp);
    }




    public static Complex[] triPoints(int i,int j,int k) {
	Complex[] w=corridor(k); 
	int[] m={1,0,1};
	Complex u=cr(j);
	Complex v=Complex.times(u,w[i]);
	return(triPoints(m[i],v));
    }

    public static int classify(Complex z) {
	Path2D.Double gp=tri(0,new Complex(0,0));
	if(gp.contains(z.x,z.y)==false) return(-1);
	return(0);
    }

    public static Complex[][] specialSegments() {
	Complex[][] W=new Complex[9][2];
	Complex[] w=new Complex[3];
	for(int i=0;i<3;++i) w[i]=Octahedron.cr(i);
	for(int i=0;i<3;++i) {
	    Complex w1=w[i];
	    int j=(i+1)%3;
	    int k=(i+2)%3;
	    Complex w2=Complex.plus(w[j],w[k]);
	    w2=Complex.times(w2,new Complex(.5,0));
	    W[i][0]=new Complex(w1);
	    W[i][1]=new Complex(w2);
	}

	Complex[] zz=new Complex[3];
	for(int i=0;i<3;++i) {
	    int j=(i+1)%3;
	    int k=(i+2)%3;
	    zz[i]=Complex.plus(w[j],w[k]);
            zz[i]=Complex.times(zz[i],new Complex(.5,0));
	}   
	W[3][0]=new Complex();
	W[3][1]=new Complex(zz[1]);
	W[4][0]=new Complex();
	W[4][1]=new Complex(zz[2]);
	W[5][0]=new Complex();
	W[5][1]=new Complex(zz[0]);

	W[6][0]=new Complex(w[0]);
	W[6][1]=new Complex(w[1]);
	W[7][0]=new Complex(w[1]);
	W[7][1]=new Complex(w[2]);
	W[8][0]=new Complex(w[2]);
	W[8][1]=new Complex(w[0]);
	return(W);
    }

    public static int nearSpecial(Complex z) {
	int index=-1;
	Complex[][] W=specialSegments();
	double min=100;
	for(int i=0;i<9;++i) {
	    double dist=Complex.distanceToLine(z,W[i][0],W[i][1]);
	    if(dist<min) {
		min=dist;
		index=i;
	    }
	}
	return(index);
    }

    public static Complex projectToLine(int i,Complex z) {
	Complex[][] W=specialSegments();
	Complex w=Complex.projectToLine(z,W[i][0],W[i][1]);
	return(w);
    }


    public static Complex[] scaffold1(int i,Complex z) {
	Complex[][] W=Octahedron.bisectorList(z);
	int j=(i+3)%12;
	Complex X=Vector.findCross2(W[i][0],W[i][1],W[j][0],W[j][1]);
	Complex[] Z={new Complex(W[i][0]),new Complex(W[j][0]),X};
	return(Z);
    }


    public static Complex[] scaffold2(int i,Complex z) {
	double s3=Math.sqrt(3);
	Complex[] W=new Complex[2];


	if(i==0) {
	    W[0]=new Complex(-.5,-s3/2);
	    W[1]=new Complex(-2,s3);
	}
	if(i==1) {
	    W[0]=new Complex(-2,-s3);
	    W[1]=new Complex(-.5,s3/2);
	}

	if(i==2) {
	    W[0]=new Complex(-2,0);
	    W[1]=new Complex(-.5,3*s3/2);
	}

	if(i==3) {
	    W[0]=new Complex(-2,s3);
	    W[1]=new Complex(1,s3);
	}

	if(i==4) {
	    W[0]=new Complex(-.5,s3/2);
	    W[1]=new Complex(2.5,s3/2);
	}

	if(i==5) {
	    W[0]=new Complex(-.5,3*s3/2);
	    W[1]=new Complex(1,0);
	}

	if(i==6) {
	    W[0]=new Complex(1,s3);
	    W[1]=new Complex(2.5,-s3/2);
	}

	if(i==7) {
	    W[0]=new Complex(2.5,s3/2);
	    W[1]=new Complex(1,-s3);
	}

	if(i==8) {
	    W[0]=new Complex(1,0);
	    W[1]=new Complex(-.5,-3*s3/2);
	}

	if(i==9) {
	    W[0]=new Complex(2.5,-s3/2);
	    W[1]=new Complex(-.5,-s3/2);
	}

	if(i==10) {
	    W[0]=new Complex(1,-s3);
	    W[1]=new Complex(-2,-s3);
	}

	if(i==11) {
	    W[1]=new Complex(-2,0);
	    W[0]=new Complex(-.5,-3*s3/2);
	}
	return(W);
    }

    public static Path2D.Double rim(int k) {
	if(k==0) return(rim0());
	if(k==1) return(rim1());
	if(k==2) return(rim2());
	if(k==3) return(rim3());
	return(null);
    }

    public static Path2D.Double rim1() {
	Complex[] Z=rim();
	for(int i=0;i<6;++i) Z[i]=Octahedron.anti(Z[i]);
	Complex[] W={Z[5],Z[0],Z[1],Z[4]};
	Path2D.Double gp=GraphicsHelp.toPath(W);
	return(gp);
    }
    public static Path2D.Double rim2() {
	Complex[] Z=rim();
	for(int i=0;i<6;++i) Z[i]=Octahedron.anti(Z[i]);
	Complex[] W={Z[1],Z[2],Z[3],Z[0]};
	Path2D.Double gp=GraphicsHelp.toPath(W);
	return(gp);
    }

    public static Path2D.Double rim3() {
	Complex[] Z=rim();
	for(int i=0;i<6;++i) Z[i]=Octahedron.anti(Z[i]);
	Complex[] W={Z[3],Z[4],Z[5],Z[2]};
	Path2D.Double gp=GraphicsHelp.toPath(W);
	return(gp);
    }

    public static Path2D.Double rim0() {
	double s3=Math.sqrt(3);
	Complex[] Z=rim();
	Path2D.Double gp=GraphicsHelp.toPath(Z);
	return(gp);
    }

    public static Complex[] rim() {
	double s3=Math.sqrt(3);
	Complex[] Z=new Complex[6];
	Z[0]=new Complex(2.5,-s3/2);
	Z[1]=new Complex(2.5,+s3/2);
	Z[2]=Complex.times(Z[0],cr(1));
	Z[3]=Complex.times(Z[1],cr(1));
	Z[4]=Complex.times(Z[0],cr(2));
	Z[5]=Complex.times(Z[1],cr(2));
	return(Z);
    }

}


